// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#include <QTest>

int main(int argc, char *argv[])
{
    QTest::qSleep(10);
    return 0;
}
