// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#include <QDebug>
#include "button.h"

Button::Button(QWidget *parent)
    : QPushButton(parent)
{
}

void Button::animateClick()
{
    qDebug() << "Extra code goes here...";
    QPushButton::animateClick();
}
